import logging
from functools import wraps
import datetime

log = logging.getLogger('facebook_api_log')  # pick up the log


def write_to_log(func):
    """Decorator function to write to log"""
    @wraps(func)
    def wrapper(*args, **kwargs):
        """wrapper to put details into log"""
        args_str = [x for x in args]
        kwargs_str = ', '.join([':'.join([str(j) for j in i]) for i in kwargs.items()])
        log.info(f'running function: {func.__name__}, module: {func.__module__} @ {datetime.datetime.now()}')
        log.debug(f'args: {args_str}')
        log.debug(f'kwargs: {kwargs_str}')
        return func(*args, **kwargs)
    return wrapper




from sqlalchemy import create_engine, MetaData, Table
from sqlalchemy.orm import sessionmaker

from ge_sm.common.secrets import rs_usr, rs_pwd, rs_dsn

class RedshiftDBConnection:

    """creates a SQLAlchemy engine, metadata and session objects"""

    def __init__(self, usr=rs_usr, pwd=rs_pwd, svr=rs_dsn, schema='chaajaa', connect_args={'sslmode': 'prefer'}):

        self.usr = usr
        self.pwd = pwd
        self.svr = svr
        self.schema = schema
        self.connect_args = connect_args
        self.db_string = f'redshift+psycopg2://{self.usr}:{self.pwd}@{self.svr}:5439'
        if self.connect_args:
            self.engine = create_engine(self.db_string, connect_args=self.connect_args)
        else:
            self.engine = create_engine(self.db_string)
        self.metadata = MetaData(bind=self.engine, schema=self.schema)
        self.Session = sessionmaker(bind=self.engine)
        self.session = self.Session()


    def __repr__(self):
        return f'{self.__class__.__name__}(usr="{self.usr}", pwd="{self.pwd}", svr="{self.svr}", ' \
            f'schema="{self.schema}")'

    @staticmethod
    def autoload_tables(table_name: str, metadata, engine, schema: str):
        """
        load metadata from pre-defined DB structure
        Also force id column to be primary key
        :param metadata:
        :param table_name: required; TableName to be defined (must exist in DB)
        :return: metadata table definition
        """
        t = Table(table_name, metadata, autoload=True, autoload_with=engine, schema=schema)
        return t

if __name__ == '__main__':
    conn = RedshiftDBConnection()
    print(conn.__dict__)


import pandas as pd
import datetime

from ge_sm.google_analytics_code.settings.chaa_jaa_settings import chaajaauser
from ge_sm.google_analytics_code.api_code.google_analytics_api import GoogleAnalyticsAPI
from ge_sm.common.helper_funcs import ga_dim_met_create

def get_report(ga, dimensions, viewid, startdate, enddate, metrics, *orders):
    request = dict(viewId=viewid, pageSize=100000, dateRanges={'startDate': startdate, 'endDate': enddate},
                   dimensions=dimensions, metrics=metrics)
    if len(orders) > 0:
        request['orderBys'] = orders[0][0]
    r = ga.analytics.reports().batchGet(body={'reportRequests': request}).execute()
    return r

def construct_user_report(stdate, eddate, rpttype):

    # Get GA API
    ga = GoogleAnalyticsAPI()
    df_dict = {}
    view = chaajaauser['view']; viewid = chaajaauser['viewId'] #; pgfilter = chaajaauser['pgfilter']
    cjdict = chaajaauser[rpttype]['dimdict']
    metnames = []; mets = []
    # get metric details
    for m_key, m_val in chaajaauser[rpttype]['mets'].items():
        metnames.append(m_key)
        mets.append(m_val)
    # Loop over all dimensions in dimension list and form data frame for each different metric
    for rpt_name, dimdict in cjdict.items():
        dimnames = []; dims = []
        for d_key, d_val in dimdict.items():
            dimnames.append(d_key)
            dims.append(d_val)
        # turn into pairs of dimensions / metrics
        dimensions, metrics = ga_dim_met_create(dims, mets)
        #list of data to turn into dataframe
        data_list = []
        # get report from GA
        resp = get_report(ga, dimensions, viewid, stdate, eddate, metrics) #, pgfilter) # Note additional sorting thing for page views
        # break down response object into parts that are required
        metric_headers = resp['reports'][0]['columnHeader']['metricHeader']['metricHeaderEntries']  # list of metrics
        dim_headers = resp['reports'][0]['columnHeader']['dimensions']  # columns dimensions
        data = resp['reports'][0]['data']['rows']
        for d in data:
            # zip takes iteration and returns pairs, which can then be make key / value in dictionary
            x = dict(zip(dim_headers, d['dimensions']))
            y = dict(zip([d['name'] for d in metric_headers], d['metrics'][0]['values']))
            y['view'] = view
            y['viewid'] = viewid
            data_list.append({**x, **y})
        # set dictionary value to be dataframe
        df_ga_user = pd.DataFrame(data_list)
        # print(rpt_name, dimdict, df_ga_user.head().to_string())
        # sort out names and order of columns
        df_ga_user = df_ga_user[['ga:' + d for d in dims] + ['ga:' + m for m in mets]]
        df_ga_user.columns = dimnames + metnames
        # strip down pages if required
        if rpttype == 'pages':
            df_ga_user['Page Path'] = df_ga_user['Page Path'].str.slice(0, 255)
            df_ga_user['Page Title'] = df_ga_user['Page Title'].str.slice(0, 255)
        if rpttype == 'events':
            df_ga_user['Event Label'] = df_ga_user['Event Label'].str.slice(0, 255)

        # Convert kadence id to numeric if there at all
        if 'KadenceId' in df_ga_user.columns:
            # Get rid of alex and agnes test kadence ids
            df_ga_user = df_ga_user[(df_ga_user['KadenceId'] !='alextest') &
                                    (df_ga_user['KadenceId'] !='agnestest')]
            df_ga_user['KadenceId'] = df_ga_user['KadenceId'].astype(str).str.\
                slice(0, 6, 1).astype('int64')
        df_dict[rpt_name] = df_ga_user

    return df_dict

def make_signedup_df(df_client_pg, df_wagtail_pg, df_client_ur, df_kdids):

    # just get individual dim 1 and 2 for client and kadence ids
    # get the various

    # get unique kadence ids for pages
    df_client_list_pg = df_client_pg.drop_duplicates(subset = 'KadenceId', keep = 'first')
    df_client_list_pg = df_client_list_pg[['KadenceId', 'ClientId']]

    # get unique wagtail ids for pages
    df_wagtail_list_pg = df_wagtail_pg.drop_duplicates(subset = 'KadenceId', keep = 'first')
    df_wagtail_list_pg = df_wagtail_list_pg[['KadenceId', 'WagtailId']]

    # get video events
    df_client_list_ur = df_client_ur[['KadenceId', 'Event Label']][(df_client_ur['Event Category'] == 'Video Interactions')]
    # get unique video titles
    df_client_list_ur = df_client_list_ur.drop_duplicates()
    # count number unique videos seen
    df_client_list_ur = df_client_list_ur[['KadenceId', 'Event Label']].groupby('KadenceId', as_index=False).count()
    df_client_list_ur.columns = ['KadenceId', 'Unique Video Count']
    print(df_client_list_ur.to_string())

    # # Merge both in outer join with kadence id files
    df_match = pd.merge(df_kdids, df_client_list_pg, how = 'left', on = 'KadenceId')

    df_match = pd.merge(df_match, df_wagtail_list_pg, how = 'left', on = 'KadenceId')

    # add in video count
    df_match = pd.merge(df_match, df_client_list_ur, how = 'left', on = 'KadenceId')

    #add in date analysis was run
    df_match['Date'] = pd.to_datetime(datetime.date.today(), format='%Y-%m-%d')
    return df_match[['Date', 'KadenceId', 'WagtailId', 'ClientId', 'Unique Video Count']]



if __name__ == '__main__':
    # print(settings)
    report_control()


from ge_sm.common.secrets import *

pg_info = {
    "pageId": os.environ.setdefault('CHAA_JAA_PAGEID', '****'),
    "facebookPageName": os.environ.setdefault('CHAA_JAA_FACEBOOKPAGENAME', '****'),
    "gePageName": os.environ.setdefault('CHAA_JAA_GEPAGENAME', '****'),
    "pageApp": os.environ.setdefault('CHAA_JAA_PAGEAPP', '****'),
    "access_token": os.environ.setdefault('CHAA_JAA_ACCESS_TOKEN', '****')
}

page_output_details = {'Date': ['date','1a',None],
'Page ID':['pageId',1,None],
'Page name':['pageName',2,None],
'Total reach':['page_impressions_unique',3,'Metric'],
'Paid reach':['page_impressions_paid_unique',4,'Metric'],
'Organic reach':['page_impressions_organic_unique',5,'Metric'],
'Viral reach':['page_impressions_viral_unique',6,'Metric'],
'Perc reach organic':['Perc reach organic',7,None], ## organic reach/  total reach
'Perc reach paid':['Perc reach paid',8,None], ## paid reach/  total reach
'Perc reach viral':['Perc reach viral',9,None], ## viral reach/  total reach
'Total Impressions':['page_impressions',10,'Metric'],
'Paid impressions':['page_impressions_paid',11,'Metric'],
'Organic impressions':['page_impressions_organic',12,'Metric'],
'Viral impressions':['page_impressions_viral',13,'Metric'],
'Total clicks':['page_consumptions',13,'Metric'],
'Unique clicks':['page_consumptions_unique',13,'Metric'],
'Engagement rate: unique clicks/reach':['Engagement_rate',14,None], ## Unique clicks / total reach
'Engaged users':['page_engaged_users',15,'Metric'],
'Page post engagements':['page_post_engagements',16,'Metric'],
'New likes':['page_fan_adds_unique',17,'Metric'],
'Unlikes':['page_fan_removes_unique',18,'Metric'],
'Net likes':['Net_Likes',19,None], ## New likes - Unlikes
'Page likes':['page_positive_feedback_by_type',20,'like'],
'Page shares':['page_positive_feedback_by_type',21,'link'],
'Page comments':['page_positive_feedback_by_type',22,'comment'],
'Page answers':['page_positive_feedback_by_type',22,'answer'],
'Page claims':['page_positive_feedback_by_type',22,'claim'],
'Page rsvps':['page_positive_feedback_by_type',22,'rsvp'],
'Page others':['page_positive_feedback_by_type',22,'other'],
'Total video views':['page_video_views',23,'Metric'],
'Unique video views':['page_video_views_unique',24,'Metric'],
'Total 10s video views':['post_video_views_10s',25,'Metric'],
'Total 30s video views':['page_video_complete_views_30s',26,'Metric']}


demo_groups = {'F.13-17': ['Female','13-17'], 'F.18-24':['Female','18-24'], 'F.25-34':['Female','25-34'], 'F.35-44':['Female','35-44'], 
'F.45-54':['Female','45-54'], 'F.55-64':['Female','55-64'], 'F.65+':['Female','65+'], 
'M.13-17':['Male','13-17'], 'M.18-24':['Male','18-24'], 'M.25-34':['Male','25-34'], 'M.35-44':['Male','35-44'], 
'M.45-54':['Male','45-54'], 'M.55-64':['Male','55-64'], 'M.65+':['Male','65+'], 
'U.13-17':['Male','13-17'], 'U.18-24':['Male','18-24'], 'U.25-34':['Male','25-34'], 'U.35-44':['Male','35-44'], 
'U.45-54':['Male','45-54'], 'U.55-64':['Male','55-64'], 'U.65+':['Male','65+']}


#This is dictionary of post metrics: GE name, FB name, order for dataframe, type:
#:None is field, metric is metric, else is key of dictionary for this metric (when data comes out as dictionary, e.g., post_activity_by_action_type
post_output_details = {'Date': ['date','1a',None],
'Post release date' : ['created_time',1,None],
'Post Id' : ['postId',2,None],
'Page ID' : ['pageId',3,None],
'Page name' : ['pageName',4,None],
'Post name' : ['message',5,None],
'url': ['permalink_url', '5a', None],
'total_comments' : ['total_comments',6,None],
'total_reactions' : ['total_reactions',7,None],
'total_shares' : ['total_shares',8,None],
'Post engagement fan' : ['post_engaged_fan',31, 'Metric'],
'Post engaged users' : ['post_engaged_users',9,'Metric'],
'Total post impressions' : ['post_impressions',10,'Metric'],
'Total post impressions organic': ['post_impressions_organic',11,'Metric'],
'Total post impressions paid': ['post_impressions_paid',13,'Metric'],
'Total post impressions viral' : ['post_impressions_viral',16,'Metric'],
'Total post reach' : ['post_impressions_unique',15,'Metric'],
'Organic reach' : ['post_impressions_organic_unique',12,'Metric'],
'Paid reach' : ['post_impressions_paid_unique',14,'Metric'],
'Viral reach' : ['post_impressions_viral_unique',16,'Metric'],
'Post Engagements' : ['post_engagements',16,None], ## sum of Post clicks and Post reactions, comments, shares
'Negative actions' : ['post_negative_feedback',17,'Metric'],
'Post likes' : ['post_reactions_like_total',18,'Metric'],
'Post loves' : ['post_reactions_love_total',18,'Metric'],
'Post wow' : ['post_reactions_wow_total',18,'Metric'],
'Post haha' : ['post_reactions_haha_total',18,'Metric'],
'Average video view time' : ['post_video_avg_time_watched',19,'Metric'],
'Total video views' : ['post_video_views',20,'Metric'],
'10 second video views' : ['post_video_views_10s',21,'Metric'],
'Unique 10 second video views' : ['post_video_views_10s_unique',22,'Metric'],
'Total unique video views' : ['post_video_views_unique',23,'Metric'],
'Total reactions, comments, shares' : ['post_activity',25,None], ##sum of below
'Post reactions' : ['post_activity_by_action_type',25,'like'],
'Post comments' : ['post_activity_by_action_type',24,'comment'],
'Post shares' : ['post_activity_by_action_type',26,'share'],
'Video duration':['vid_duration',23,None],
'30 second video views' : ['page_video_complete_views_30s',27,'Metric'],
'Unique 30 second video views' : ['page_video_complete_views_30s_unique',28,'Metric'],
'Post clicks' : ['post_clicks',30,'Metric'],
'Post clicks unique' : ['post_clicks_unique',30,'Metric'],
'Post clicks_video' : ['post_clicks_by_type',30,'video play'],
'Post clicks_photo' : ['post_clicks_by_type',30,'photo view'],
'Post clicks_other' : ['post_clicks_by_type',30,'other'],
'Post clicks unique_video' : ['post_clicks_by_type_unique',30,'video play'],
'Post clicks unique_photo' : ['post_clicks_by_type_unique',30,'photo view'],
'Post clicks unique_other' : ['post_clicks_by_type_unique',30,'other']}






"""just a bunch of functions that make life easier"""
import json
import datetime
import calendar

def ga_dim_met_create(dims, mets):
    #make GA dimension / metric list
    return [{'name': 'ga:' + d} for d in dims], [{'expression': 'ga:' + m} for m in mets]

def derive_dates(dt):
    """derive 1st of the current month and 1st of the next month from date padded in"""
    s = datetime.datetime.strftime((datetime.datetime(dt.year, dt.month, 1)), '%Y-%m-%d')
    e = datetime.datetime.strftime((datetime.datetime(dt.year, dt.month, dt.day+1)), '%Y-%m-%d')
    return s, e

def derive_dates_posts(dt):
    """parameters to query data for relative to today, return start and end of current month"""
    s = datetime.datetime.strftime((datetime.datetime(dt.year, dt.month, 1)), '%Y-%m-%d')
    e = datetime.datetime.strftime(dt.replace(day=calendar.monthrange(dt.year, dt.month)[1]),
                                   '%Y-%m-%d')
    return s, e

# functions to help with JSON
def print_json(j):
    print(json.dumps(j, indent=4, sort_keys=True))

def write_json(p: list, pth: str, fn: str):
    with open(f'./{pth}/{fn}.json', 'w') as f:
        json.dump(p, f)

def read_json(pth: str, fn: str):
    try:
        with open(f'{pth}/{fn}.json', 'r', encoding='utf-8') as f:
            d = json.load(f)
    except FileNotFoundError:
        with open(f'{fn}.json', 'r', encoding='utf-8') as f:
            d = json.load(f)

    return d




from ge_sm.common.secrets import *


chaajaauserpagesdimdict = {
	'ga_user_pages_C': {'DateTime': 'dateHourMinute', 'ClientId': 'dimension1', 'Page Path': 'pagePath', 'Page Title': 'pageTitle'},
	'ga_user_pages_CK': {'DateTime': 'dateHourMinute', 'ClientId': 'dimension1', 'KadenceId': 'dimension2', 'Page Path': 'pagePath', 'Page Title': 'pageTitle'},
	'ga_user_pages_CW': {'DateTime': 'dateHourMinute', 'ClientId': 'dimension1', 'WagtailId': 'dimension3', 'Page Path': 'pagePath', 'Page Title': 'pageTitle'},
	'ga_user_pages_CKW': {'DateTime': 'dateHourMinute', 'ClientId': 'dimension1', 'KadenceId': 'dimension2', 'WagtailId': 'dimension3', 'Page Path': 'pagePath', 'Page Title': 'pageTitle'}
}

chaajaausereventsdimdict = {
	'ga_user_events_C': {'DateTime': 'dateHourMinute', 'ClientId': 'dimension1', 'Event Category': 'eventCategory', 'Event Label': 'eventLabel'},
	'ga_user_events_CK': {'DateTime': 'dateHourMinute', 'ClientId': 'dimension1', 'KadenceId': 'dimension2', 'Event Category': 'eventCategory', 'Event Label': 'eventLabel'},
	'ga_user_events_CW': {'DateTime': 'dateHourMinute', 'ClientId': 'dimension1', 'WagtailId': 'dimension3', 'Event Category': 'eventCategory', 'Event Label': 'eventLabel'},
	'ga_user_events_CKW': {'DateTime': 'dateHourMinute', 'ClientId': 'dimension1', 'KadenceId': 'dimension2', 'WagtailId': 'dimension3', 'Event Category': 'eventCategory', 'Event Label': 'eventLabel'}
}


chaajaauser = {
	'view': os.environ.setdefault('CHAAJAAUSER_VIEW', '****'),
	'viewId': os.environ.setdefault('CHAAJAAUSER_VIEWID', '****'),
	'pages': {
		'dimdict': chaajaauserpagesdimdict,
		'mets': {'Time on page': 'timeonpage', 'Page views': 'pageviews'}},
	'events': {
		'dimdict': chaajaausereventsdimdict,
		'mets': {'Total Events': 'totalevents', 'Unique Events': 'uniqueevents'}}
}






import pandas as pd
import logging

from ge_sm.facebook_code.api_code.facebook_API import FaceBookPageData
#from common.logger import write_to_log
from ge_sm.facebook_code.settings.chaa_jaa_settings import pg_info, page_output_details as po, demo_groups


log = logging.getLogger('facebook_api_log')

#@write_to_log
def get_page_insights(fb, page_metrics, page_metrics_splits, sd, ed, period) -> pd.DataFrame:

    # get insights data in similar manner to post
    pg_metrics_str = ','.join(page_metrics)
    fb.query_insights(metrics = pg_metrics_str, since = sd, until = ed, period = period)
    all_insights = []
    date_len = len(fb.insight_data['data'][0]['values'])
    #loop over all elements in date range
    for i in range(date_len):
        d = {}
        for m in page_metrics:
            d[m] = 0
        data = fb.insight_data['data']
        for p in data:
            d['end_time'] = p['values'][i]['end_time']
            result = p['values'][i]['value']
            # sort out rubbish dictionary output, i.e. if it one of the dictionaries...
            if p['name'] in page_metrics_splits:
                # get details
                met_list = page_metrics_splits[p['name']]
                # go over all possible split values for this metric
                # if in metric value then set output to be this, else set to 0
                for t in met_list:
                    d[p['name'] + '_' + t] = 0
                    if t in result: d[p['name'] + '_' + t] = result[t]
            else:
                d[p['name']] = result
        # append data as list of dictionaries for data frame
        all_insights.append(d)
    return pd.DataFrame(all_insights)

#@write_to_log
def construct_insight_data(stdate, eddate, period): #mnth: int = None, yr: int = None):

    #sort out metrics from imported page details
    #see post comments for more info here
    pm_list = []; page_metrics_order = []; pm_split = {}; page_metrics_names = []
    for p, q in po.items():
        #get GE names and order
        page_metrics_names.append(p)
        #if a metric / split metric add to post_metrics
        if q[2] !=None: pm_list.append(q[0])
        #If a split metric then add to list of splits / add new split metric
        if q[2] != 'Metric' and q[2] != None:
            page_metrics_order.append(q[0] + '_' + q[2])
            if q[0] in pm_split:
                pm_split[q[0]].append(q[2])
            else:
                pm_split[q[0]] = [q[2]]
        else:
            page_metrics_order.append(q[0])
    fb_page_obj = FaceBookPageData(pagetoken = pg_info['access_token'], page = pg_info['pageId'])
    df_page_insights = get_page_insights(fb_page_obj, pm_list, pm_split, stdate, eddate, period)

    # print(df_page_insights.tail().to_string())
    #add in pageid and page name
    df_page_insights['pageId'] = pg_info['pageId']
    df_page_insights['pageName'] = pg_info['facebookPageName']
    df_page_insights['date'] = pd.to_datetime(df_page_insights['end_time'].str.slice(0, 10), format='%Y-%m-%d',
                                      errors='coerce') - pd.Timedelta('1 day')
    #do various page metric calcs
    df_page_insights['Perc reach organic'] = \
        df_page_insights['page_impressions_organic_unique'] / df_page_insights['page_impressions_unique']
    df_page_insights['Perc reach viral'] = \
        df_page_insights['page_impressions_viral_unique'] / df_page_insights['page_impressions_unique']
    df_page_insights['Perc reach paid']  = \
        df_page_insights['page_impressions_paid_unique'] / df_page_insights['page_impressions_unique']
    df_page_insights['Engagement_rate'] = \
        df_page_insights['page_consumptions_unique'] / df_page_insights['page_impressions_unique']
    df_page_insights['Net_Likes'] = \
        df_page_insights['page_fan_adds_unique'] - df_page_insights['page_fan_removes_unique']
    df_page_insights = df_page_insights[page_metrics_order]
    df_page_insights.columns = page_metrics_names
    return df_page_insights


def get_fb_demo_data(sd, ed, period = 'day'): #mnth: int = None, yr: int = None):
    """queries page level data for the current month"""
    #Use dictionary of possible demos
    fb = FaceBookPageData(pagetoken = pg_info['access_token'], page = pg_info['pageId'])
    demo_mets = ['page_impressions_by_age_gender_unique']
    fb.query_insights(metrics=demo_mets, since = sd, until = ed, period = period)


    demo_data = fb.insight_data['data'][0]['values']

    demo_list = []
    #Loop over all values for dataframe
    for d in demo_data:
        dg_dict = d['value']
        end_time = d['end_time']
        # go over all demo groups
        for dg_key, dg_item in demo_groups.items():
            pd_dict = {}
            pd_dict['end_time'] = end_time
            pd_dict['Page name'] = pg_info['facebookPageName']
             # set gender / age group to be zero for each
            pd_dict['Gender'] = dg_item[0]
            pd_dict['Age'] = dg_item[1]
            pd_dict['Impressions'] = 0
            # if this key in fb dictionary, then set value
            if dg_key in dg_dict:
                pd_dict['Impressions'] = dg_dict[dg_key]
                # add to dictionary
            demo_list.append(pd_dict)
    df_demos = pd.DataFrame(demo_list)
    df_demos['Date'] = pd.to_datetime(df_demos['end_time'].str.slice(0, 10), format='%Y-%m-%d',
                                      errors='coerce') - pd.Timedelta('1 day')
    #print(df_demos.to_string())
    df_order = ['Date', 'Page name', 'Gender', 'Age', 'Impressions']
    return df_demos[df_order]


if __name__ == '__main__':

    # to load historic data
    populate_historic()







import facebook
import logging

from ge_sm.facebook_code.api_code.facebook_secrets import long_access_token

#is this used?
#from common.logger import write_to_log

log = logging.getLogger('facebook_api_log')

class FaceBookAPIConnection:
    """base connection to FB API"""

    def __init__(self, access_token=long_access_token, version="3.1"):
        self.access_token = access_token
        self.version = version
        self.my_account_info = None
        self.graph = facebook.GraphAPI(access_token=self.access_token, version=self.version)

    def __repr__(self):
        return f'{self.__class__.__name__}(access_token="{self.access_token}", version="{self.version}")'

    def get_object(self):
        """get info about 'my' account"""
        self.my_account_info = self.graph.get_object('me/accounts')

class FaceBookPageData(FaceBookAPIConnection):
    """API connection to page related queries"""
    def __init__(self, *, pagetoken=None, page=None):
        super().__init__()
        self.pagetoken = pagetoken
        self.page = page
        self.graph = facebook.GraphAPI(access_token=self.pagetoken, version=self.version)
        self.insight_data = None

    def __repr__(self):
        return f'{self.__class__.__name__}(pagetoken="{self.pagetoken}", page="{self.page}")'


    def query_insights(self, connection_name='insights', metrics=None, since='2019-01-01', until='2019-02-02',
                       period='week'):
        """query insights data - sets a set of default metrics is nothing is passes in"""
        if metrics is None:
            metrics = 'page_fans, page_fan_adds, page_fan_removes, page_impressions_unique, ' \
                      'page_impressions_organic_unique, page_impressions_paid_unique, page_impressions_viral_unique'
        self.insight_data = self.graph.get_connections(id=self.page, connection_name=connection_name, metric=metrics,
                                                       since=since, until=until, period=period)

class FaceBookPostData(FaceBookAPIConnection):
    """API connection to page related queries"""
    def __init__(self, *, pagetoken=None, page=None):
        super().__init__()
        self.pagetoken = pagetoken
        self.page = page
        self.graph = facebook.GraphAPI(access_token=self.pagetoken, version=self.version)
        self.post_ids, self.posts, self.post_data, self.post_connections = None, None, None, None
        self.post_insights, self.post_attributes = None, None


    def query_post_ids(self, fields=None, limit=50):
        """return post ids and limit to the value of 'limit'"""
        if fields is None:
            fields = f'posts.fields(title, created_time).limit({limit})'
        self.post_ids = self.graph.get_object(id=self.page, fields=fields)

    def query_posts(self):
        """retrieve post date from feed - prespective is post posted by 'me' on 'my feed' on 'my page'"""
        self.post_data = self.graph.get_connections(id=self.page, connection_name='feed')

    def query_all_posts(self, fields=None, since='2018-08-01', until='2018-08-31'):
        """Queried all posts to return data an id of each post and returns generator object"""
        if fields is None:
            #fields = 'created_time, message, type'
            fields = 'created_time, permalink_url, properties'
        self.posts = self.graph.get_all_connections(id=self.page, connection_name='posts', fields=fields,
                                                    since=since, until=until)

    def query_post_connections(self, postid):
        """returns connections to the post"""
        self.post_connections = self.graph.get_object(id=postid, fields="""message, comments.summary(total_count), 
                                                      reactions.summary(total_count), shares""")

    def query_post_attributes(self, postid, connection_name='comments', include_hidden=True,
                              order='reverse_chronological', filter='stream', summary='total_count'):
        self.post_attributes = self.graph.get_connections(id=postid, connection_name=connection_name,
                                                          include_hidden=include_hidden, order=order,
                                                          filter=filter, summary=summary)
																

    def query_post_insights(self, postid, connection_name='insights', metrics='post_activity_by_action_type',
                            datepreset='yesterday', period='lifetime', show_description_from_api_doc=True):
        self.post_insights = self.graph.get_connections(id=postid, connection_name=connection_name, metric=metrics,
                                                        datepreset=datepreset,period=period,
                                                        show_description_from_api_doc=show_description_from_api_doc)













import json
from pathlib import Path
from ge_sm.common.secrets import *


pathname = Path(os.path.join(os.path.abspath('')))

channelsdict = {
	os.environ.get('YT_CHANNEL', ''): {
		'name': 'Chajaa',
		'analyticsdat': 'analytics_CJ.dat',
		'playlist': os.environ.get('YT_PLAYLIST', ''),
		'youtubedat':  'youtube_CJdat.dat'
	}
}

#Metric details
std_metric_details = {'Views': ['views','1a','Metric'],
'Watch time':['watchtime',1,None], ## views * avg view time
'Avg view duration':['averageViewDuration',1,'Metric'],
'Avg view percentage':['averageViewPercentage',2,'Metric'],
'Comments':['comments',3,'Metric'],
'Likes':['likes',4,'Metric'],
'Dislikes':['dislikes',5,'Metric'],
'Shares':['shares',6,'Metric'],
'Subscribers':['subscribers',7, None], ## subs gained - subs lost
'Subscribers gained':['subscribersGained',7,'Metric'], 
'Subscribers lost':['subscribersLost',8,'Metric'], 
'Annotation Clicks':['annotationClicks',9,'Metric'], 
'Annotation clickable impressions':['annotationClickableImpressions',10,'Metric'],
'Annotation click through rate':['annotationClickThroughRate',11,'Metric']}

vid_details = [('ChName','Channel name'), ('ChId', 'Channel ID'), ('video', 'Video Id'), 
('videoname','Video name'), ('videourl', 'URL'), ('videopub','Date published'), ('estimatedMinutesWatched', 'Est Minutes Watched')]



from ge_sm.common.secrets import *

#facebook
app_id = os.environ.setdefault('FACEBOOK_APP_ID', '****')
access_token = os.environ.setdefault('FACEBOOK_ACCESS_TOKEN', '****')


secret = os.environ.setdefault('FACEBOOK_APP_SECRET', '****')
app_token = os.environ.setdefault('FACEBOOK_APP_TOKEN', '****')

pg_token = os.environ.setdefault('FACEBOOK_PG_TOKEN', '****')
long_access_token = os.environ.setdefault('FACEBOOK_LONG_ACCESS_TOKEN', '****')



from ge_sm.common.database_connection import RedshiftDBConnection
from sqlalchemy import Table, Column, Date, String, BIGINT, Float
import csv


def upload_with_pandas(df, tblname):
    con = RedshiftDBConnection()
    df.to_sql(name = tblname, con = con.engine, index = False, schema = 'chaajaa', if_exists = 'append')

def create_table_from_csv(tblname, fielddefs):
    con = RedshiftDBConnection()
    con.schema = 'chaajaa'
    meta = con.metadata
    tb = Table(tblname, meta)
    # open csv file of field names
    with open('./common/' + fielddefs) as csv_file:
        dtmap = [r for r in csv.reader(csv_file)]
    csv_file.close()
    # conversion dictionary for data dtypes
    typedict = {'date': Date, 'double precision': Float, 'bigint': BIGINT, 'varchar(256)': String,
                'varchar(1000)': String}
    for d in dtmap:
        print(d[0], d[3])
        tb.append_column(Column(d[0], typedict[d[3]]))
    meta.create_all(con.engine)

def delete_all_data(tblname):
    # just get rid of current data in given table
    con = RedshiftDBConnection()
    con.schema = 'chaajaa'
    meta = con.metadata
    tb = Table(tblname, meta)
    con.engine.execute(tb.delete())


if __name__ == '__main__':
    # historic_load(create_table=False, tblname='fb_page_metrics_monthly')
    con = RedshiftDBConnection()



import datetime
import pandas as pd
import shutil
import os
from dateutil.relativedelta import relativedelta
# from common.logger import write_to_log
from ge_sm.facebook_code.data_creation.query_pages import construct_insight_data, get_fb_demo_data  # derive_dates
from ge_sm.facebook_code.data_creation.query_posts import construct_post_data
from ge_sm.facebook_code.data_creation.query_ads import construct_ad_data

from ge_sm.youtube_code.data_creation.query_reports import run_video_report, run_channel_report, run_demo_report

from ge_sm.google_analytics_code.data_creation.query_reports import construct_user_report, make_signedup_df

from ge_sm.common.create_database_tables import upload_with_pandas, delete_all_data
from ge_sm.control_details import reportlist

# put in path name for secrets et al as running from batch file
upload_dir = 'ge_sm/sm-upload'
path_dir = os.path.abspath('')
pathname = os.path.join(path_dir, upload_dir)

# from common.create_database_tables import update_monthly_table

# # set logger
# logging.basicConfig(filename='facebook_api_log',
#                     filemode='a',
#                     format='%(asctime)s - %(levelname)s - %(module)-20s: %(message)s',
#                     level=logging.DEBUG)
# log = logging.getLogger('facebook_api_log')

# @write_to_log
def main(s, page_start, e):

    user_stdt = '2019-06-12'
    post_start = '2019-06-01'

    # note adding in page / demo start dates as it only seems to like <=90 days (!)
    # demo_start = '2019-08-01'
    #
    # # # # Get various dataframes and add to dictionary
    ## FBADs
    reportlist['fb_ads']['df'] = construct_ad_data(post_start, e)

    # Other FB
    reportlist['fb_demos']['df'] = get_fb_demo_data(page_start, e, 'days_28')
    reportlist['fb_pages_day']['df'] = construct_insight_data(page_start, e, 'day')
    reportlist['fb_pages_week']['df'] = construct_insight_data(page_start, e, 'week')
    reportlist['fb_pages_days_28']['df'] = construct_insight_data(page_start, e, 'days_28')
    reportlist['fb_posts']['df'] = construct_post_data(post_start, e)

    ## YT
    reportlist['yt_channels']['df'] = run_channel_report(post_start, e)
    reportlist['yt_demos']['df'] = run_demo_report(post_start, e)
    reportlist['yt_video']['df'] = run_video_report(post_start, e)
    #
    # # # # Import kadence Id files
    # cancelled it no longer needed
    # if os.path.exists(pathname + '/Kadence_Ids.csv'):
    #     df_kadence = pd.read_csv(pathname + '/Kadence_Ids.csv')
    #     df_kadence.columns = ['KadenceId', 'Centre']
    #     reportlist['ga_signed_up']['df'] = make_signedup_df(reportlist['ga_user_pages_CK']['df'],
    #                                                     reportlist['ga_user_pages_CKW']['df'],
    #                                                     reportlist['ga_user_events_CK']['df'],
    #                                                     df_kadence)
    # user_pages_dfs = construct_user_report(user_stdt, e, 'pages')

    # ## GA
    # reportlist['ga_user_pages_C']['df'] = user_pages_dfs['ga_user_pages_C']
    # reportlist['ga_user_pages_CK']['df'] = user_pages_dfs['ga_user_pages_CK']
    # reportlist['ga_user_pages_CW']['df'] = user_pages_dfs['ga_user_pages_CW']
    # reportlist['ga_user_pages_CKW']['df'] = user_pages_dfs['ga_user_pages_CKW']
    # user_events_dfs = construct_user_report(user_stdt, e, 'events')
    # reportlist['ga_user_events_C']['df'] = user_events_dfs['ga_user_events_C']
    # reportlist['ga_user_events_CK']['df'] = user_events_dfs['ga_user_events_CK']
    # reportlist['ga_user_events_CW']['df'] = user_events_dfs['ga_user_events_CW']
    # reportlist['ga_user_events_CKW']['df'] = user_events_dfs['ga_user_events_CKW']
    #


    #
    # # # # # # # Loop over each element in report list
    for rpt, data in reportlist.items():
        # if rpt != 'fb_ads' and rpt != 'fb_posts' and rpt[:2] == 'fb':
        if rpt[:2] == 'yt' or rpt[:2] == 'fb':
        # try:
            print(rpt)
            # First make table if need (normally keep commented out)
            #     create_table_from_csv(data['tablename'], data['tablestructure'])
            # Copy old .csv files into same location to keep them for future checking
            csvlen = len(data['outputcsvloc']) - 4
            if os.path.exists(pathname + data['outputcsvloc']):
                shutil.copy2(pathname + data['outputcsvloc'], pathname +data['outputcsvloc'][:csvlen] + '_Old.csv')
            # # Delete all data from given table
            # delete_all_data(data['tablename'])
            # temp thing to sort out FB page issue on 3 months
            df = data['df']
            if data['FBPAGE'] == 'Yes' and os.path.exists(pathname + data['outputcsvloc']):
                # get historic data for this FB object
                olddata_df = pd.read_csv(pathname + data['outputcsvloc'])
                # knock out current month then merge with downloaded data
                olddata_df['Date'] = pd.to_datetime(olddata_df['Date'].astype(str).str.slice(0, 10), format = '%Y-%m-%d')
                olddata_df = olddata_df[olddata_df['Date'] < page_start]
                df = pd.concat([olddata_df, df], ignore_index = True, sort = False)
            print(df.to_string())

            os.makedirs(os.path.dirname(pathname + data['outputcsvloc']), exist_ok=True)
            # Export dataframe to csv
            if not os.path.exists(pathname + data['outputcsvloc']):
                open(pathname + data['outputcsvloc'], 'a').close()

            df.to_csv(pathname + data['outputcsvloc'], index = False)
            # Upload to data platform
            # upload_with_pandas(df, data['tablename'])
            # except:
            #     pass

if __name__ == '__main__':
    today = datetime.datetime.now()
    tm = datetime.datetime.strftime(today + datetime.timedelta(days=2), "%Y-%m-%d")
    td = datetime.datetime.strftime(today, "%Y-%m-%d")
    yd = datetime.datetime.strftime(today + datetime.timedelta(days=-1), "%Y-%m-%d")
    print(yd, td, tm)

    # keep eye on month for now
    if not os.environ.get('DYNAMIC_DATES'):
        main(
            os.environ.get('START_DATE', '2019-06-01'),
            os.environ.get('END_DATE', '2019-08-01'),
            tm
        )
    else:
        months = os.environ.get('REPORT_MONTHS', 2)
        main(
            today.replace(day=1).date().strftime('%Y-%m-%d'),
            (today - relativedelta(months=months)).replace(day=1).strftime('%Y-%m-%d'),
            tm
        )



import pandas as pd
import logging

#from common.logger import write_to_log
from ge_sm.facebook_code.api_code.facebook_API import FaceBookPostData
from ge_sm.facebook_code.settings.chaa_jaa_settings import pg_info, post_output_details as po

log = logging.getLogger('facebook_api_log')

def get_post_insights(fb, post_iterator, post_metrics, post_metrics_split) -> pd.DataFrame:
    '''use post connection and attributes methods, yields total shares, reactions and comments
    plus insights for a laugh :)'''

    post_metrics_str = ','.join(post_metrics)
    all_insights = []
    for post in post_iterator:
        #get all post connections / attributes, post metrics
        fb.query_post_connections(post['id'])
        fb.query_post_attributes(post['id'])
        fb.query_post_insights(post['id'], metrics = post_metrics_str)

        # construct basic post dictionary
        d = {'postId': post['id'],
             'created_time': post['created_time'],
             'message' : None,
             'permalink_url': post['permalink_url'],
             'vid_duration': 0,
             #'comments': None,
             'total_reactions': 0,
             'total_shares': 0,
             'total_comments': 0}

        # TODO should log empty values
        if fb.post_connections.get('message') is not None:
            d['message'] = fb.post_connections['message']
        #Find video length
        if 'properties' in post:
            for p in post['properties']:
                if p['name'] == 'Length': d['vid_duration'] =  p['text']

        #now do insights
        #set default value up here
        for m in post_metrics:
            d[m] = 0
        #so now only will overwrite missing ones!
        for p in fb.post_insights['data']:
            #get value for the metric
            result = p['values'][0]['value']
            # sort out rubbish dictionary output, i.e. if it one of the dictionaries...
            if p['name'] in post_metrics_split:
                #get details
                met_list = post_metrics_split[p['name']]
                #go over all possible split values for this metric
                #if in metric value then set output to be this, else set to 0
                for t in met_list:
                    d[p['name'] + '_' + t] = 0
                    if t in result: d[p['name'] + '_' + t] = result[t]
            else:
                d[p['name']] = result

        #append data as list of dictionaries for data framw
        all_insights.append(d)
    return pd.DataFrame(all_insights)

#@write_to_log
def construct_post_data(stdt, eddt):

    # get all posts for all pages in this date range
    #Get fb post object
    fb_post_obj = FaceBookPostData(pagetoken = pg_info['access_token'], page = pg_info['pageId'])
    #call method to get all post ids / dates
    fb_post_obj.query_all_posts(since = stdt, until = eddt)
    # this is a list of dictionarys, date then post_id
    post_iterator = fb_post_obj.posts

    #sort out metrics from imported page details
    pm_list = []; post_metrics_order = []; pm_split = {}; post_metrics_names = []
    for p, q in po.items():
        #get GE names and order
        post_metrics_names.append(p)
        #if a metric / split metric add to post_metrics
        if q[2] !=None: pm_list.append(q[0])
        #If a split metric then add to list of splits / add new split metric
        if q[2] != 'Metric' and q[2] != None:
            post_metrics_order.append(q[0] + '_' + q[2])
            if q[0] in pm_split:
                pm_split[q[0]].append(q[2])
            else:
                pm_split[q[0]] = [q[2]]
        else:
            post_metrics_order.append(q[0])

    df_post_insights = get_post_insights(fb_post_obj, post_iterator, pm_list, pm_split)
    #add in pageid and page name
    df_post_insights['pageId'] = pg_info['pageId']
    df_post_insights['pageName'] = pg_info['facebookPageName']
    df_post_insights['date'] = pd.to_datetime(df_post_insights['created_time'].str.slice(0, 10), format='%Y-%m-%d',
                                      errors='coerce')
    #this was in milliseconds
    df_post_insights['post_video_avg_time_watched'] = df_post_insights['post_video_avg_time_watched'] / 1000
    df_post_insights['post_activity'] = df_post_insights['post_activity_by_action_type_like'] + \
                                        df_post_insights['post_activity_by_action_type_comment'] + \
                                        df_post_insights['post_activity_by_action_type_share']
    df_post_insights['post_engagements'] = df_post_insights['post_activity'] + df_post_insights['post_clicks']
    df_post_insights = df_post_insights[post_metrics_order]
    df_post_insights.columns = post_metrics_names
    post_metrics_names.remove('Post release date')
    #get rid of long post name
    df_post_insights['Post name'] = df_post_insights['Post name'].str.slice(0, 255)
    return df_post_insights[post_metrics_names]




from ge_sm.common.secrets import *

ad_account = {
	'accountId': os.environ.setdefault('AD_ACCOUNT_ACCOUNTID', '****'),
	'accountName': os.environ.setdefault('AD_ACCOUNT_ACCOUNTNAME', '****'),
}

chaa_jaa_ad_account = {
	'accountId': os.environ.setdefault('CHAA_JAA_ACCOUNTID', '****'),
	'accountName': os.environ.setdefault('CHAA_JAA_ACCOUNTNAME', '****'),
}


#all_campaigns = [{'campaignId' : 23843354108040284, 'campaignName': 'ID - NI - Hero Videos'}, #
				#{'campaignId' : 23843354102160284, 'campaignName': 'ID - NI - Real Eating - Post_Imp'}, 
				#{'campaignId' : 23843319823660284, 'campaignName': 'ID - Real Eating - Reach'}] 

chaa_jaa_campaigns = [{
	'campaignId': os.environ.setdefault('CHAA_JAA_CAMPAIGNID', '****'),
	'campaignName': os.environ.setdefault('CHAA_JAA_CAMPAIGNNAME', '****')
}]

# 23843416588070205

campaign_output_details = {
	'Campaign Id': ['campaign_id', '1a', 'Adset'],
	'Campaign name': ['campaignName', '1a', None],
	'Ad Set ID': ['id', 1, 'Adset'],
	'Ad set name': ['name', 2, 'Adset'],
	'Delivery status': ['status', 2, None],
	'Lifetime budget': ['lifetime_budget', 2, 'Adset'],
	'Reach': ['reach', 2, 'Metric'],
	'Impressions': ['impressions', 2, 'Metric'],
	'Frequency': ['frequency', 2, 'Metric'],
	'Result type': ['optimization_goal', 3, 'Adset'],
	'Results': ['results', 3, None],
	'Cost per result': ['cpr', 3, None],
	'Amount spent': ['spend', 3, 'Metric'],
	'Starts': ['start_time', 3, 'Adset'],
	'Ends': ['end_time', 3, 'Adset'],
	'Clicks': ['clicks', 3, 'Metric'],
	'Unique Clicks': ['unique_clicks', 3, 'Metric'],
	'Cost per click': ['cpc', 3, 'Metric'],
	'Cost per 1000 impressions': ['cpm', 3, 'Metric'],
	'Cost per 1000 reach': ['cpp', 3, 'Metric'],
	'Cost per unique click': ['cost_per_unique_click', 3, 'Metric'],
	'Cost per unique inline click': ['cost_per_unique_inline_link_click', 3, 'Metric'],
	'Cost per thruplay': ['cost_per_thruplay', 3, 'Metric'],
	'Photo View': ['actions', 3, 'photo_view'],
	'Cost per photo view': ['cost_per_action_type', 3, 'photo_view'],
	'Shares': ['actions', 3, 'post'],
	'Cost per share': ['cost_per_action_type', 3, 'post'],
	'Likes': ['actions', 3, 'like'],
	'Cost per like': ['cost_per_action_type', 3, 'like'],
	'Comments': ['actions', 3, 'comment'],
	'Cost per comment': ['cost_per_action_type', 3, 'comment'],
	'Link Clicks': ['actions', 3, 'link_click'],
	'Cost per link click': ['cost_per_action_type', 3, 'link_click'],
	'Post engagements': ['actions', 3, 'post_engagement'],
	'Cost per post engagement': ['cost_per_action_type', 3, 'post_engagement'],
	# 'Messages': ['actions', 3, 'onsite_conversion.messaging_conversation_start'],
	# 'Cost per message': ['cost_per_action_type', 3, 'onsite_conversion.messaging_conversation_start'],
	'Post landing page': ['actions', 3, 'landing_page_view'],
	'Cost per landing page': ['cost_per_action_type', 3, 'landing_page_view'],
}


result_dict = {
	'POST_ENGAGEMENT': ('Post engagement', 'cost_per_action_type_post_engagement'),
	'REACH': ('Reach', 'cpp'),
	'THRUPLAY': ('Thruplay', 'cost_per_thruplay'),
	'LANDING_PAGE_VIEWS': ('Landing page views', 'cost_per_action_type_landing_page_view'),
	'LINK_CLICKS': ('Link clicks', 'cost_per_action_type_link_click'),
	'PAGE_LIKES': ('Page likes', 'cost_per_action_type_like'),
}



import os
from oauth2client import client, file, tools
from apiclient.discovery import build
import httplib2
from pathlib import Path

# put in path name for secrets et al as running from batch file
########################
pathname = Path(os.path.join(os.path.abspath('')))
########################

# key = 'AIzaSyCl41ZyJ70G9C68DTa8Yo6a-gS1Z9Pxs-g'
CLIENT_ID = os.environ.get('YT_ANALYTICS_CLIENT_ID')
CLIENT_SECRET = os.environ.get('YT_ANALYTICS_CLIENT_SECRET')
ACCESS_TOKEN = os.environ.get('YT_ANALYTICS_ACCESS_TOKEN')
R_TOKEN = os.environ.get('YT_ANALYTICS_REFRESH_TOKEN')


class YoutubeAnalyticsAPI:
    """Create a youtube analytics session using oauth2 flow
    by abstracting this away makes it easier to change the oauth2 flow in the future"""
    def __init__(self, scopes = ['https://www.googleapis.com/auth/yt-analytics.readonly'],
                 secrets_path = pathname / 'youtube_secrets_details.json',
                 analyticsdat = pathname / 'YTanalytics.dat'):
        self.scopes = scopes
        self.secrets_path = secrets_path
        self.initialize_analyticsreporting(analyticsdat)

    def __repr__(self):
        return f'{self.__class__.__name__}(scopes="{self.scopes}"", secrets_path="{self.secrets_path}"'

    def initialize_analyticsreporting(self, analyticsdat):
        credentials = client.OAuth2Credentials(
            ACCESS_TOKEN, CLIENT_ID, CLIENT_SECRET, R_TOKEN,
            None, 'https://oauth2.googleapis.com/token', None)
        http = credentials.authorize(httplib2.Http())
        self.analytics = build('youtubeAnalytics', 'v2', http=http)


class YoutubeAPI:
    """Create a youtube analytics session using oauth2 flow
    by abstracting this away makes it easier to change the oauth2 flow in the future"""
    def __init__(self, scopes = ['https://www.googleapis.com/auth/youtube.readonly',
                                 'https://www.googleapis.com/auth/yt-analytics.readonly'],
                 secrets_path = pathname / 'youtube_secrets_details.json',
                 youtubedat = pathname / 'YTanalytics.dat'):
        self.scopes = scopes
        self.secrets_path = secrets_path
        self.initialize_apireporting(youtubedat)

    def __repr__(self):
        return f'{self.__class__.__name__}(scopes="{self.scopes}"", secrets_path="{self.secrets_path}"'

    def initialize_apireporting(self, youtubedat):
        # Parse command-line arguments.
        credentials = client.OAuth2Credentials(
            ACCESS_TOKEN, CLIENT_ID, CLIENT_SECRET, R_TOKEN,
            None, 'https://oauth2.googleapis.com/token', None)
        http = credentials.authorize(httplib2.Http())
        self.reports = build('youtube', 'v3', http=http)





import pandas as pd

from ge_sm.youtube_code.settings.chaa_jaa_settings import channelsdict, std_metric_details as sm
from ge_sm.youtube_code.api_code.youtube_analytics_api import YoutubeAnalyticsAPI, YoutubeAPI

#Not using this at present
def get_allvideodetails(yt, channel):
    # # Get playlist details
    request_pl = yt.reports.playlists().list(channelId=channel, part='snippet').execute()
    playlists_ids = [i['id'] for i in request_pl['items']]
    # have two lists to deal with possible duplication of video id
    all_videos = []; all_video_ids = []
    for pl in playlists_ids:
        request = yt.reports.playlistItems().list(part = 'snippet, contentDetails', playlistId = pl).execute()
        for i in request['items']:
            v = i['snippet']
            video_pub = v['publishedAt']
            video_title = v['title']
            video_id = v['resourceId']['videoId']
            #add playlst id here
            video_pl = pl
            # deal with duplicate video ids here
            if video_id not in all_video_ids:
                all_videos.append([video_id, video_title,'https://youtu.com/embed/' + video_id, video_pub, video_pl])
                all_video_ids.append(video_id)
    return(all_videos)

def get_report(yta, channel, startdate, enddate, metrics, dimensions, *filter):
    """performs the request to get analytics data according to dims and metrics"""
    request = {'ids': channel, 'startDate': startdate, 'endDate': enddate,'maxResults': '200', 'dimensions': dimensions,
               'metrics': metrics}
    if len(filter) > 0:
        #add sort or similar from the filter parameter, note it is the first element of tuple is key, second value
        request[filter[0][0]] = filter[0][1]
    return yta.analytics.reports().query(**request).execute()

def run_video_report(sd, ed):
    """sd start date, ed enddate"""
    metrics_list = []; metrics_order = []; metrics_names = []
    for p, q in sm.items():
        #get GE names and order
        metrics_names.append(p)
        metrics_order.append(q[0])
        #if a metric / split metric add to post_metrics
        if q[2] !=None: metrics_list.append(q[0])
    stdmetrics = ','.join(metrics_list)
    # hold video details as list of data frames
    df_videos = None
    #loop over each channel
    for id in channelsdict:
        channel = channelsdict[id]
        #get name of anlaytics.dat file for this channel
        yta_dat = channel['analyticsdat']
        yt_dat = channel['youtubedat']
        yt = YoutubeAPI(youtubedat = yt_dat)
        yta = YoutubeAnalyticsAPI(analyticsdat = yta_dat)
        data_list = []
        #playlists = get_allplaylists(yt, id)
        #get all videos plus the names, id, urls
        vid_details = get_allvideodetails(yt, id)
        #get video names
        # have to dael with possible video id duplication
        df_vidnames = pd.DataFrame([{'playList': v[4], 'video': v[0], 'videoname': v[1], 'videourl': v[2], 'videopub': v[3]} for v in vid_details])
        # get api data for each channel
        resp = get_report(yta,'channel==' + id, sd, ed, 'estimatedMinutesWatched,' + stdmetrics,
        'video', ('sort','-estimatedMinutesWatched'))
        # get column headers and data
        col_headers = [d['name'] for d in resp['columnHeaders']]
        data = resp['rows']
        # loop over all data, and zip correct head to metrics value
        for d in data:
            x = (dict(zip(col_headers, d)))
            x['ChName'] = channel['name'] # name and id of channel for output
            x['ChId'] = id
            #append to data list
            data_list.append(x)
        # set dictionary values to be dataframe
        df = pd.DataFrame(data_list)
        #merge in with video names
        df = pd.merge(df, df_vidnames, how='inner', on='video')
        #add to existing dataframe
        if df_videos is None:
            df_videos = df
        else:
            df_videos = df_videos.append(df, sort=True)
            df_videos.reset_index(drop=True, inplace=True)

    #do subscriber calculation
    df_videos['subscribers'] = df_videos['subscribersGained'] - df_videos['subscribersLost']
    df_videos['watchtime'] = df_videos['views'] * df_videos['averageViewDuration'] / 60
    #sort out column headers
    df_videos = df_videos[['ChName', 'ChId', 'playList', 'video', 'videoname', 'videourl', 'videopub', 'estimatedMinutesWatched'] + metrics_order]
    #convert vid published into something sensible
    df_videos['videopub'] = pd.to_datetime(df_videos['videopub'].str.slice(0, 10), format='%Y-%m-%d',
                   errors='coerce')
    df_videos.columns = ['Channel name', 'Channel ID', 'PlayList', 'Video Id', 'Video name', 'Url', 'Date published', 'Est Minutes Watched'] + metrics_names
    return(df_videos)
#
def run_channel_report(sd, ed):
    """sd start date, ed enddate"""
    metrics_list = []; metrics_order = []; metrics_names = []
    for p, q in sm.items():
        #get GE names and order
        metrics_names.append(p)
        metrics_order.append(q[0])
        #if a metric / split metric add to post_metrics
        if q[2] !=None: metrics_list.append(q[0])
    stdmetrics = ','.join(metrics_list)

    df_channels = None
    #loop over each channel
    for id in channelsdict:
        channel = channelsdict[id]
        #get name of anlaytics.dat file for this channel
        ytadat = channel['analyticsdat']
        yta = YoutubeAnalyticsAPI(analyticsdat = ytadat)
        resp = get_report(yta, 'channel==' + id, sd, ed, stdmetrics, 'day')
        data_list = []
        # break down response object into parts that are required
        col_headers = [d['name'] for d in resp['columnHeaders']]
        data = resp['rows']
        for d in data:
            x = (dict(zip(col_headers, d)))
            x['ChName'] = channel['name'] # name and id of channel for output
            x['ChId'] = id
            data_list.append(x)
        df = pd.DataFrame(data_list)
        # add to existing dataframe
        if df_channels is None:
            df_channels = df
        else:
            df_channels = df_channels.append(df, sort=True)
            df_channels.reset_index(drop=True, inplace=True)

    # do subscriber calculation
    df_channels['subscribers'] = df_channels['subscribersGained'] - df_channels['subscribersLost']
    df_channels['watchtime'] = df_channels['views'] * df_channels['averageViewDuration'] / 60
    # sort out column headers
    df_channels = df_channels[['day', 'ChName', 'ChId'] + metrics_order]
    df_channels.columns = ['Date', 'Channel name','Channel ID'] + metrics_names
    return(df_channels)

def run_demo_report(sd, ed):
    """sd start date, ed enddate"""
    # hold video details as list of data frames
    df_demos = None
    #loop over each channel
    for id in channelsdict:
        channel = channelsdict[id]
        #get name of anlaytics.dat file for this channel
        ytadat = channel['analyticsdat']
        yta = YoutubeAnalyticsAPI(analyticsdat=ytadat)
        data_list = []
        # get api data for each channel
        resp = get_report(yta,'channel==' + id, sd, ed, 'viewerPercentage',
        'ageGroup,gender', ('sort','ageGroup,gender'))
        # get column headers and data
        col_headers = [d['name'] for d in resp['columnHeaders']]
        data = resp['rows']
        # loop over all data, and zip correct head to metrics value
        for d in data:
            x = (dict(zip(col_headers, d)))
            x['ChName'] = channel['name'] # name and id of channel for output
            x['ChId'] = id
            x['Date'] = sd
            #append to data list
            data_list.append(x)
        # set dictionary values to be dataframe
        df = pd.DataFrame(data_list)
        #add to existing dataframe
        if df_demos is None:
            df_demos = df
        else:
            df_demos = df_demos.append(df, sort=True)
            df_demos.reset_index(drop=True, inplace=True)
    #sort ouf month here for date
    df_demos['Date'] = pd.to_datetime(df_demos['Date'].str.slice(0, 10), format='%Y-%m',
                   errors='coerce')
    #sort out column headers
    df_demos = df_demos[['Date', 'ChName', 'ChId', 'gender', 'ageGroup', 'viewerPercentage']]
    return(df_demos)





#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup, find_packages, find_namespace_packages
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

install_requires = [
    'facebook_sdk==3.1.0',
    'facebook_business==6.0.4',
    'httplib2==0.13.1',
    'pandas==0.25.1',
    'SQLAlchemy==1.3.8',
    'google_api_python_client==1.7.11',
    'oauth2client==4.1.3',
    'sqlalchemy-redshift==0.7.5',
    'psycopg2'
]
setup(
    name='ge_sm',
    version='0.0.18',
    description="A python applications.",
    long_description=readme,
    author="Praekelt.org",
    author_email='dev@praekelt.org',
    packages=find_packages(),  # include all packages under src

    include_package_data=True,
    install_requires=install_requires,
    license="BSD",
    zip_safe=False,
    keywords='ge_sm',
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.6',
    ]
)



import pandas as pd

from facebook_business import FacebookSession, FacebookAdsApi
from facebook_business.adobjects.campaign import Campaign as AdCampaign
from facebook_business.adobjects.adaccount import AdAccount

# This is api key file location
# going forward move perhaps to api_code folder?
from ge_sm.facebook_code.api_code.facebook_secrets import long_access_token as access_token, secret as app_secret, app_id
from ge_sm.facebook_code.settings.chaa_jaa_ad_account_settings import chaa_jaa_ad_account
from ge_sm.facebook_code.settings.chaa_jaa_ad_account_settings import campaign_output_details as co
from ge_sm.facebook_code.settings.chaa_jaa_ad_account_settings import result_dict

def get_campaigns(ad_account, api_session):
    """gets campaign ids for specific ad account"""
    FacebookAdsApi.set_default_api(api_session)
    acc = AdAccount(ad_account)
    campaign_objs = [campaign for campaign in acc.get_campaigns(fields=[AdCampaign.Field.name])]
    all_campaigns = [c for c in campaign_objs]
    return campaign_objs, all_campaigns

def construct_adset_df(ad_dict_list, met_order, met_split):
    adset_data = []
    #loop over all ad in list of dictionaries
    for ad in ad_dict_list:
        adfields = ad['ad_fields']; admetrics = ad['ad_metrics']
        # default set all fields to be None
        ad_data = {}
        for m in met_order:
            ad_data[m] = None
        ad_data['campaignId'] = ad['campaignId']
        ad_data['campaignName'] = ad['campaignName']
        ad_data['status'] = ad['status']
        #now set all relevant values in ad field
        for ad_key, ad_val in adfields.items():
            #print(ad_key, ad_val)
            ad_data[ad_key] = ad_val
        # if there is any sort of metric data
        if admetrics != None:
            for in_key, in_val in admetrics.items():
                # If split metric then go inside of list
                if in_key in met_split:
                    # Look for various wanted metrics (note this is a list of dictionaries bizarrely)
                    for m in in_val:
                        # make name as in dictionary field (note key of dict is action_type)
                        metname = in_key + '_' + m['action_type']
                        # If one of the desired fields, set dataframe dictionary value
                        if metname in met_order:
                            # Only add if this field > 0
                            if float(m['value']) > 0 : ad_data[metname] = float(m['value'])
                # just for cost per thru play .. very odd
                elif in_key == 'cost_per_thruplay':
                     # weirdly it's a list with a value, no idea what's going on here
                    ad_data['cost_per_thruplay'] = in_val[0]['value']
                 # Else just set value if key there
                else:
                    ad_data[in_key] = in_val
            #sort out liftime budget
            if ad_data['lifetime_budget'] == None: ad_data['lifetime_budget'] = 0
            ad_data['lifetime_budget'] = float(ad_data['lifetime_budget']) / 100
            # sort out the nonsense about the result thing
            # and add in the cpr if can work out what it is
            ad_data['cpr'] = 0
            # get actual optimisation goal
            temp = ad_data['optimization_goal']
            # if it is one of the known goals
            if temp in result_dict:
                #if there is a cost per dict value somewhere
                if ad_data[result_dict[temp][1]] != None:
                    # set cpr to be this cost per result
                    ad_data['cpr'] = float(ad_data[result_dict[temp][1]])
                    # if this is non-zero try to back-calculate the result; needed as seem to be
                    # missing the actual value half the time!
                    if ad_data['cpr'] > 0:
                        ad_data['results'] = int(float(ad_data['spend']) / ad_data['cpr'])
                ad_data['optimization_goal'] = result_dict[temp][0]
            # add all ad data as new element of a list
            adset_data.append(ad_data)
    return pd.DataFrame(adset_data)

def construct_ad_data(stdate, eddate):

    # here we get fields / metrics we need
    ad_field_list = []; ad_met_list = []
    ad_metrics_order = []; ad_split = {}; ad_metrics_names = []
    for p, q in co.items():
        # append GE names for dataframe
        ad_metrics_names.append(p)
        #if a field / metric / split metric add to post_metrics
        if q[2] != None:
            # If property of adset, not insight then add to adset field list
            if q[2] == 'Adset':
                ad_field_list.append(q[0])
                ad_metrics_order.append(q[0])
            # If insight then add to adset insight list
            elif q[2] == 'Metric':
                ad_met_list.append(q[0])
                ad_metrics_order.append(q[0])
            else:
                # Here we deal with the metric splits in usual way (see post stuff)
                ad_metrics_order.append(q[0] + '_' + q[2])
                ad_met_list.append(q[0])
                if q[0] in ad_split:
                    ad_split[q[0]].append(q[2])
                else:
                    ad_split[q[0]] = [q[2]]
        else:
            #If one of the non field / metrics then just add to order list of df
            ad_metrics_order.append(q[0])

    # Some sort of crummy FB session stuff
    session = FacebookSession(app_id, app_secret, access_token)
    api = FacebookAdsApi(session)
    FacebookAdsApi.set_default_api(api)

    # Here we get all the ads for this account
    acc = AdAccount(chaa_jaa_ad_account['accountId'])
    # get all campaign objects
    campaign_objs = [campaign for campaign in acc.get_campaigns(fields=['name', 'status', 'start_time','stop_time'])]
    compaigns = {}
    # get some properties of each campaign
    for c in campaign_objs:
        if len(c) > 4:
            compaigns[c['id']] = (c['name'], c['status'], c['start_time'],c['stop_time'])
        else:
            compaigns[c['id']] = (c['name'], c['status'], c['start_time'])

    # get all adsets for this ad account
    alladsets = [adset for adset in acc.get_ad_sets(fields = ad_field_list)]

    adset_details = []  ## get ad details first
    # parameters for the adsets
    insight_params = {'time_range': {'since': stdate, 'until': eddate}}
    for ad in alladsets:
        # get all insights for this particular addset id accoun
        camp = compaigns[ad['campaign_id']]
        # print(camp)
        # get campaign details
        ad_dict = {'campaignId': ad['campaign_id'], 'campaignName': camp[0],'status': camp[1],
                   'ad_fields': ad, 'ad_metrics': None}
        # ge ad insights
        ad_insights = ad.get_insights(fields = ad_met_list, params = insight_params)
        # Now get the relevant insights
        if len(ad_insights) > 0: ad_dict['ad_metrics'] = ad_insights[0]
        # print(ad_dict)
        adset_details.append(ad_dict)
    df_ad_insights = construct_adset_df(adset_details, ad_metrics_order, ad_split)
    # now do some tidying of data
    df_ad_insights = df_ad_insights[ad_metrics_order]
    df_ad_insights.columns = ad_metrics_names

    #sort out the date fields
    df_ad_insights['Starts'] = pd.to_datetime(df_ad_insights['Starts'].str.slice(0, 10),
                                               format='%Y-%m-%d',  errors='coerce')
    df_ad_insights['Ends'] = pd.to_datetime(df_ad_insights['Ends'].str.slice(0, 10),
                                               format='%Y-%m-%d',  errors='coerce')
    #
    # Now do something about the very odd delivery thing:
    # if active and > today - day (today is 12 at night time)
    print(df_ad_insights.to_string())
    df_ad_insights.loc[(df_ad_insights['Delivery status'] == 'ACTIVE') &
           (df_ad_insights['Ends'] >= (pd.to_datetime('today') - pd.Timedelta('7 days'))), 'temp'] = 'Recently Completed'
    # if active and ends within 7 days
    df_ad_insights.loc[(df_ad_insights['Delivery status'] == 'ACTIVE') &
           (df_ad_insights['Ends'] >= pd.to_datetime('today') - pd.Timedelta('1 day')), 'temp'] = 'Active'
    # if active and ends after 7 days
    df_ad_insights.loc[(df_ad_insights['Delivery status'] == 'ACTIVE') &
           (df_ad_insights['Ends'] < (pd.to_datetime('today') - pd.Timedelta('7 days'))), 'temp'] = 'Completed'
    # else inactive
    df_ad_insights.loc[df_ad_insights['Delivery status'] != 'ACTIVE', 'temp'] = 'Inactive'
    df_ad_insights['Delivery status'] = df_ad_insights['temp']
    df_ad_insights = df_ad_insights.drop('temp', axis=1)
    return df_ad_insights

if __name__ == '__main__':

    main()


import os
import argparse
from oauth2client import client, file, tools
from googleapiclient.discovery import build
from httplib2 import Http
from pathlib import Path
# put in path name for secrets et al as running from batch file
########################
pathname = Path(os.path.join(os.path.abspath('')))
########################


class GoogleAnalyticsAPI:
    """Create a google analytics session using oauth2 flow
    by abstracting this away makes it easier to change the oauth2 flow in the future"""
    def __init__(self, scopes = ['https://www.googleapis.com/auth/analytics.readonly'],
                 secrets_path = pathname / 'google_analytics_secrets_details.json',
                 discovery_uri = ('https://analyticsreporting.googleapis.com/$discovery/rest')):
        self.scopes = scopes
        self.secrets_path = secrets_path
        self.discovery_uri = discovery_uri
        self.initialize_analyticsreporting()

    def __repr__(self):
        return f'{self.__class__.__name__}(scopes="{self.scopes}"", secrets_path="{self.secrets_path}",' \
               f'discovery_uri="{self.discovery_uri}"'


    def initialize_analyticsreporting(self):
        # Parse command-line arguments.
        parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                         parents=[tools.argparser])
        flags = parser.parse_args([])
        # Set up a Flow object to be used if we need to authenticate.
        flow = client.flow_from_clientsecrets(self.secrets_path, scope=self.scopes,
                                              message=tools.message_if_missing(self.secrets_path))
        storage = file.Storage(pathname + 'analyticsreporting.dat')
        credentials = storage.get()
        if credentials is None or credentials.invalid:
            credentials = tools.run_flow(flow, storage, flags)
        self.analytics = build('analytics', 'v4', http=credentials.authorize(http=Http()),
                               discoveryServiceUrl=self.discovery_uri) #PB COMMENT, NOT SURE WHAT LAST LINE MEANS?


if __name__ == '__main__':

    ga = GoogleAnalyticsAPI()
    print(ga)



reportlist = {
    'fb_ads': {'tablename': 'tbl_fb_ads', 'outputcsvloc': '/facebook_code/data_csv/Chaajaa_FB_Ads.csv', 'tablestructure': 'FB_Ads_DataTypes.csv', 'FBPAGE': 'No'},
    'fb_demos': {'tablename': 'tbl_fb_demos', 'outputcsvloc': '/facebook_code/data_csv/Chaajaa_FB_Demos.csv', 'tablestructure': 'FB_Demos_DataTypes.csv', 'FBPAGE': 'Yes'},
    'fb_pages_day': {'tablename': 'tbl_fb_pages_day', 'outputcsvloc': '/facebook_code/data_csv/Chaajaa_FB_Pages_day.csv', 'tablestructure': 'FB_Pages_DataTypes.csv', 'FBPAGE': 'Yes'},
    'fb_pages_week': {'tablename': 'tbl_fb_pages_week', 'outputcsvloc': '/facebook_code/data_csv/Chaajaa_FB_Pages_week.csv', 'tablestructure': 'FB_Pages_DataTypes.csv', 'FBPAGE': 'Yes'},
    'fb_pages_days_28': {'tablename': 'tbl_fb_pages_days_28', 'outputcsvloc': '/facebook_code/data_csv/Chaajaa_FB_Pages_days_28.csv', 'tablestructure': 'FB_Pages_DataTypes.csv', 'FBPAGE': 'Yes'},
    'fb_posts': {'tablename': 'tbl_fb_posts', 'outputcsvloc': '/facebook_code/data_csv/Chaajaa_FB_Posts.csv', 'tablestructure': 'FB_Posts_DataTypes.csv', 'FBPAGE': 'No'},
    'yt_channels': {'tablename': 'tbl_yt_channels', 'outputcsvloc': '/youtube_code/data_csv/Chaajaa_YT_Channels.csv', 'tablestructure': 'YT_Channels_DataTypes.csv', 'FBPAGE': 'No'},
    'yt_demos' :{'tablename': 'tbl_yt_demos', 'outputcsvloc': '/youtube_code/data_csv/Chaajaa_YT_Demos.csv', 'tablestructure': 'YT_Demos_DataTypes.csv', 'FBPAGE': 'No'},
    'yt_video':{'tablename': 'tbl_yt_videos', 'outputcsvloc': '/youtube_code/data_csv/Chaajaa_YT_Videos.csv', 'tablestructure': 'YT_Videos_DataTypes.csv', 'FBPAGE': 'No'},
    'ga_user_pages_C':{'tablename': 'tbl_ga_user_pages_c', 'outputcsvloc': '/google_analytics_code/data_csv/Chaajaa_GA_User_Pages_C.csv', 'tablestructure': 'GA_User_Pages_C_DataTypes.csv', 'FBPAGE': 'No'},
    'ga_user_pages_CK':{'tablename': 'tbl_ga_user_pages_ck', 'outputcsvloc': '/google_analytics_code/data_csv/Chaajaa_GA_User_Pages_CK.csv', 'tablestructure': 'GA_User_Pages_CK_DataTypes.csv', 'FBPAGE': 'No'},
    'ga_user_pages_CW':{'tablename': 'tbl_ga_user_pages_cw', 'outputcsvloc': '/google_analytics_code/data_csv/Chaajaa_GA_User_CPages_W.csv', 'tablestructure': 'GA_User_Pages_CW_DataTypes.csv', 'FBPAGE': 'No'},
    'ga_user_pages_CKW':{'tablename': 'tbl_ga_user_pages_ckw', 'outputcsvloc': '/google_analytics_code/data_csv/Chaajaa_GA_User_Pages_CKW.csv', 'tablestructure': 'GA_User_Pages_CKW_DataTypes.csv', 'FBPAGE': 'No'},
    'ga_user_events_C':{'tablename': 'tbl_ga_user_events_c', 'outputcsvloc': '/google_analytics_code/data_csv/Chaajaa_GA_User_Events_C.csv', 'tablestructure': 'GA_User_Events_C_DataTypes.csv', 'FBPAGE': 'No'},
    'ga_user_events_CK':{'tablename': 'tbl_ga_user_events_ck', 'outputcsvloc': '/google_analytics_code/data_csv/Chaajaa_GA_User_Events_CK.csv', 'tablestructure': 'GA_User_Events_CK_DataTypes.csv', 'FBPAGE': 'No'},
    'ga_user_events_CW':{'tablename': 'tbl_ga_user_events_cw', 'outputcsvloc': '/google_analytics_code/data_csv/Chaajaa_GA_User_Events_CW.csv', 'tablestructure': 'GA_User_Events_CW_DataTypes.csv', 'FBPAGE': 'No'},
    'ga_user_events_CKW':{'tablename': 'tbl_ga_user_events_ckw', 'outputcsvloc': '/google_analytics_code/data_csv/Chaajaa_GA_User_Events_CKW.csv', 'tablestructure': 'GA_User_Events_CKW_DataTypes.csv', 'FBPAGE': 'No'},
    'ga_signed_up':{'tablename': 'tbl_ga_signup', 'outputcsvloc': '/google_analytics_code/data_csv/SignUp.csv', 'tablestructure': 'GA_SignUp_DataTypes.csv', 'FBPAGE': 'No'}}


